from setuptools import setup

setup(
    name='NlpToolkit-Corpus',
    version='1.0.0',
    packages=['Corpus'],
    url='https://github.com/olcaytaner/Corpus-Py',
    license='',
    author='olcaytaner',
    author_email='olcaytaner@isikun.edu.tr',
    description='Corpus library'
)
